;==============================================================================
; Display object's color in the concole
pro Rectangle::PrintColor

self->GetProperty, color=color
print,'_____________________________'
print,'My color is:   R=',color[0],'    G=',color[1],'    B=',color[2]

end


;==============================================================================
; Display object's perimeter in the concole
pro Rectangle::PrintPerimeter

print,'_____________________________'
print,'My perimeter is: ',self.perimeter

end


;==============================================================================
; Display object's area in the concole
pro Rectangle::PrintArea

print,'_____________________________'
print,'My area is: ',self.area

end


;==============================================================================
; Calculate perimeter given length and width
pro Rectangle::CalculatePerimeter

self->GetProperty, width=width, length=length
self.perimeter = 2*(width + length)

end


;==============================================================================
; Calculate area given length and width
pro Rectangle::CalculateArea

self->GetProperty, width=width, length=length
Self.area = width*length

end


;==============================================================================
; Retrieve object properties
; - This is commonly referred to as an accessor method in OOP
;   since they provide "access" to an object's properties
pro Rectangle::GetProperty,area=area,length=length,perimeter=perimeter,width=width,color=color

if (arg_present(area)) then $
    area = self.area

if (arg_present(length)) then $
    length = self.length

if (arg_present(perimeter)) then $
    perimeter = self.perimeter

if (arg_present(width)) then $
    width = self.width

if (arg_present(color)) then $
    color = self.color

end


;==============================================================================
; Set object properties
; - This is commonly referred to as a mutator method in OOP 
;   since they alter an object's properties.
; - Note: area and perimeter are not set directly - they depend on length property!
pro Rectangle::SetProperty, width=width, length=length, color=color

if (n_elements(width) gt 0 && width gt 0.0) then begin
    self.width = width
    self->CalculateArea
    self->CalculatePerimeter
endif

if (n_elements(length) gt 0 && length gt 0.0) then begin
    self.length = length
    self->CalculateArea
    self->CalculatePerimeter
endif

if (n_elements(color) eq 3) then begin
    ; color must be a 3-element vector
    self.color = color
endif

end


;==============================================================================
; Initialization method.
; Referred to as a Constructor in OOP since it helps construct objects of this class
function Rectangle::Init, _REF_EXTRA=etc

; set default values for object's data members and ensure that they all
; internally consistent!
self.width = 1.0
self.length = 1.0
self.area = 1.0
self.perimeter = 4.0
self.color = [0,0,0]


; In case any additional properties were included in the call to obj_new(),
; pass them on to the setproperty method
if (n_elements(etc) gt 0) then self->setProperty, _EXTRA=etc

return, 1

end


;==============================================================================
; The Destructor
; Called whenever an object of this class is destroyed
pro Rectangle::Cleanup

; no heap variables to free
; or any other tidying up to do
end


;==============================================================================
; Class structure definition
; Defines the data members or variables for an object of this class
pro Rectangle__define

struct = {Rectangle $
          ,area:0.0 $
          ,perimeter:0.0 $
          ,width:0.0 $
          ,length:0.0 $
          ,color:[0B,0B,0B] $
         }

end